@@ -43,6 +43,7 @@ pub struct AssetsDatabase {
     lately_loaded: Vec<(String, AssetId)>,
     lately_unloaded: Vec<(String, AssetId)>,
     error_reporters: HashMap<TypeId, Box<dyn AssetsDatabaseErrorReporter>>,
+    first_run: bool,
 }
 
 impl AssetsDatabase {
@@ -61,6 +62,7 @@ impl AssetsDatabase {
             lately_loaded: vec![],
             lately_unloaded: vec![],
             error_reporters: Default::default(),
+            first_run: true,
         }
     }
 
@@ -273,8 +275,9 @@ impl AssetsDatabase {
         }
     }
 
-    pub fn insert(&mut self, path: &str, asset: Asset) -> AssetId {
-        let path = Self::clean_path(path);
+    pub fn insert(&mut self, asset: Asset) -> AssetId {
+        let path = asset.to_full_path();
+        let path = Self::clean_path(&path);
         let id = asset.id();
         self.lately_loaded.push((asset.protocol().to_owned(), id));
         self.assets.insert(id, (path.to_owned(), asset));
@@ -346,8 +349,12 @@ impl AssetsDatabase {
     }
 
     pub fn process(&mut self) {
-        self.lately_loaded.clear();
-        self.lately_unloaded.clear();
+        if self.first_run {
+            self.first_run = false;
+        } else {
+            self.lately_loaded.clear();
+            self.lately_unloaded.clear();
+        }
         let to_dispatch = {
             let mut bytes_read = 0;
             self.loading
@@ -368,12 +375,12 @@ impl AssetsDatabase {
                 match protocol.on_load_with_path(&path, data) {
                     AssetLoadResult::Data(data) => {
                         let asset = Asset::new(&prot, &path, data);
-                        self.insert(&asset.to_full_path(), asset);
+                        self.insert(asset);
                     }
                     AssetLoadResult::Yield(meta, list) => {
                         let list = list
                             .into_iter()
-                            .filter(|(_, path)| self.load(&path).is_ok())
+                            .filter(|(_, path)| self.load(path).is_ok())
                             .collect();
                         self.yielded.insert(path, (prot, meta, list));
                     }
@@ -407,12 +414,12 @@ impl AssetsDatabase {
                     match protocol.on_resume(meta, &list) {
                         AssetLoadResult::Data(data) => {
                             let asset = Asset::new(&prot, &path, data);
-                            self.insert(&asset.to_full_path(), asset);
+                            self.insert(asset);
                         }
                         AssetLoadResult::Yield(meta, list) => {
                             let list = list
                                 .into_iter()
-                                .filter(|(_, path)| self.load(&path).is_ok())
+                                .filter(|(_, path)| self.load(path).is_ok())
                                 .collect();
                             self.yielded.insert(path, (prot, meta, list));
                         }
